class Aggregation
{
  String _id, _embryo_id, _agg_id;
  GroupElement[] _cells;
  Group[] _computation_groups;
  int _num_cells, _add_cell_at;
  float _min_value, _max_value;
  int _selected_embryo, _selected_cell;
  PImage _thumbnail_img;
  color[] _color_map;
  int _cell_of_interest_index;
  int[] _comp_group_range_of_interest;
  
  Aggregation( String id, String e_id, String a_id, int num_cells )
  {
    _id = id;
    _embryo_id = e_id;
    _agg_id = a_id;
    _num_cells = num_cells;
        
    _selected_embryo = embryoID2Index( _embryo_id );
    _selected_cell = 0;
    
    _add_cell_at = 0;
    
    _min_value = Float.MAX_VALUE;
    _max_value = Float.MIN_VALUE;
    
    _cells = new GroupElement[_num_cells];
    _computation_groups = new Group[_num_cells];
    
    _color_map = _selected_colormap;
    _cell_of_interest_index = 0;
    
    _comp_group_range_of_interest = new int[2];
    _comp_group_range_of_interest[0] = 0;
    _comp_group_range_of_interest[1] = 9;
  }
  
  Aggregation( Aggregation other )
  {
    _id = other._id;
    _embryo_id = other._embryo_id;
    _agg_id = other._agg_id;
    _num_cells = other._num_cells;
    _add_cell_at = other._add_cell_at;
    _min_value = other._min_value;
    _max_value = other._max_value;
    
    _selected_embryo = other._selected_embryo;
    _selected_cell = other._selected_cell;
    
    _cells = new GroupElement[_num_cells];
    for ( int i = 0; i < _num_cells; i++ )
      _cells[i] = new GroupElement( other._cells[i] );
      
    _computation_groups = new Group[_num_cells];
    for ( int i = 0; i < _num_cells; i++ )
      _computation_groups[i] = new Group( other._computation_groups[i] );   
      
    _color_map = other._color_map;
    _cell_of_interest_index = other._cell_of_interest_index;
    
    _comp_group_range_of_interest = new int[2];
    _comp_group_range_of_interest[0] = other._comp_group_range_of_interest[0];
    _comp_group_range_of_interest[1] = other._comp_group_range_of_interest[1];
  }
  
  void addCell( GroupElement c, Group g )
  {
    if ( _add_cell_at < _num_cells ) 
    {
      _cells[_add_cell_at] = new GroupElement(c);
      _computation_groups[_add_cell_at] = new Group(g);
      
      _min_value = min( _min_value, c.getValue() );
      _max_value = max( _max_value, c.getValue() );
      
      ++_add_cell_at;
    }
    else println( "Trying to add a cell past array size in aggregation " + _id );
  }
  
  void setThumbnailImage( PImage img )
  {
    _thumbnail_img = createImage( img.width, img.height, RGB );
    _thumbnail_img.copy( img, 0, 0, img.width, img.height, 0, 0, img.width, img.height );
  }
  
  void initializeToRenderInGroups()
  {
    for ( int i = 0; i < _num_cells; i++ )
      for ( int j = 0; j < min(_computation_groups[i].getNumElements(), AGGREGATION_NUM_COMP_GROUP_CELLS_TO_RENDER); j++ )
        _computation_groups[i].getElement(j).render( true );
  }
  
  PImage getThumbnailImage()
  {
    return _thumbnail_img;
  }
  
  void normalizeValues()
  {
    for ( int i = 0; i < _num_cells; i++ )
      _cells[i].normalizeValue( _min_value, _max_value );
  }
  
  GroupElement getCell( int index )
  {
    return _cells[index];
  }
  
  int getNumCells()
  {
    return _num_cells;
  }
  
  String getID()
  {
    return _id;
  }
  
  String getAggID()
  {
    return _agg_id;
  }
  
  String getEmbryoID()
  {
    return _embryo_id;
  }
  
  int getSelectedEmbryo()
  {
    return _selected_embryo;
  }
  
  int getSelectedCell()
  {
    return _selected_cell;
  }
  
  float getMinValue()
  {
    return _min_value;
  }
  
  float getMaxValue()
  {
    return _max_value;
  }
  
  color[] getColorMap()
  {
    return _color_map;
  }
  
  void cellOfInterestIndex( int i )
  {
    _cell_of_interest_index = i;
  }
  
  GroupElement getCellOfInterest()
  {
    return _cells[_cell_of_interest_index];
  }
  
  int cellOfInterestIndex()
  {
    return _cell_of_interest_index;
  }
  
  Group getComputationGroup( int index )
  {
    return _computation_groups[index];
  }
  
  int[] computationGroupRangeOfInterest( int index )
  {
    return _comp_group_range_of_interest;
  }
}
