class AggregationView
{
  int _width, _height;
  int _agg_border, _agg_width, _agg_height, _agg_view_border;

  int _num_to_render;
  int _start_render_index;
  int _scroll_tick_width, _scroll_tick_interval, _scroll_length;
  int[] _scroll_pos;
  boolean _over_x_scroll;
  int _over_x_scroll_pos;

  AggregationView()
  {
    _agg_border = AGGREGATION_IMG_BORDER;
    _agg_view_border = AGGREGATION_VIEW_BORDER;

    _num_to_render = _start_render_index = 0;

    _scroll_tick_width = SCROLL_WIDTH;
    _scroll_tick_interval = _scroll_length = 0;
    _scroll_pos = new int[2];

    _over_x_scroll = false;
    _over_x_scroll_pos = 0;
  }

  void createThumbnails()
  {
    println( "Creating the aggregation thumbnails" );

    Aggregation a;
    GroupElement ge;
    Embryo e;
    PImage img;
    float v;
    float[] pos;
    int x, y;
    for ( int i = 0; i < NUM_AGGREGATIONS; i++ )
    {
      // initialize the thumbnail color
      img = createImage( _agg_width, _agg_height, RGB );
      for ( int j = 0; j < img.width*img.height; j++ )
        img.pixels[j] = AGGREGATION_FILL_COLOR;

      // color in the cells
      a = _aggregations[i];
      e = _embryos[a.getSelectedEmbryo()];
      for ( int j = 0; j < a.getNumCells(); j++ )
      {
        ge = a.getCell(j);
        v = ge.getValue();
        pos = e.getCell(e.cellID2Index(ge.getCellID())).getPos();

        x = round(pos[0]*(float)(_agg_width-1));
        y = round((1.0-pos[1])*(float)(_agg_height-1)); // translate to bottom of map....

        img.pixels[x+y*_agg_width] = getColor(a.getColorMap(), ge.getNormalizedValue());
      }

      a.setThumbnailImage( img );
    }
  }

  void setDimensions( int w, int h )
  {
    _width = w; 
    _height = h;

    _agg_width = _agg_height = _height - 2*_agg_view_border;

    setScrollDimensions();   
  }

  void setScrollDimensions()
  {
    // compute the number of aggs that can be shown
    _num_to_render = min( floor((float)_width / (float)(_agg_width+2*_agg_border)), NUM_AGGREGATIONS );
    _start_render_index = min( _start_render_index, (NUM_AGGREGATIONS-_num_to_render) );

    _scroll_length = (_agg_width + 2*_agg_border)*_num_to_render - 2*_agg_border;
    _scroll_tick_interval = floor( (float)_scroll_length / (float)NUM_AGGREGATIONS );
    _scroll_pos[0] = _agg_border;
    _scroll_pos[1] = _height;
  }

  void render()
  {
    strokeCap(SQUARE);
    int x, y, tx_offset=0, ty_offset=0;

    noSmooth();

    //
    // render the scroll
    //
    if ( _num_to_render != NUM_AGGREGATIONS)
    {
      noFill();
      stroke( SCROLL_BACKGROUND_COLOR );
      strokeWeight(  SCROLL_LINE_WEIGHT );
      line( _scroll_pos[0], _scroll_pos[1], _scroll_pos[0]+_scroll_length, _scroll_pos[1] );

      x = _scroll_pos[0];
      y = _scroll_pos[1]-_scroll_tick_width/2;
      line( x, y, x, y+_scroll_tick_width );
      x += _scroll_length;
      line( x, y, x, y+_scroll_tick_width );

      // and the scroll bar
      if ( !_over_x_scroll ) stroke( SCROLL_COLOR );
      else stroke( SCROLL_SELECTED_COLOR );
      strokeWeight(  SCROLL_BAR_LINE_WEIGHT );
      int min_pos = _scroll_pos[0] + _start_render_index*_scroll_tick_interval;
      int max_pos = _scroll_pos[0] + (_start_render_index+_num_to_render)*_scroll_tick_interval;
      line( min_pos, _scroll_pos[1], max_pos, _scroll_pos[1] );
    }

    //
    // render the embryo map
    //
    if ( _use_text_offset )
    {
      tx_offset = _aggregation_view_origin[0];
      ty_offset = _aggregation_view_origin[1];
    }   
    int y_bottom_offset = 0;
    if ( !_use_text_offset )
    { 
      ty_offset = -PDF_OFFSET;
      y_bottom_offset = PDF_OFFSET-1;
    }

    x = _agg_border;
    y = _agg_view_border;



    for ( int i = _start_render_index; i < _start_render_index+_num_to_render; i++ )
    {
      //println( _aggregations[i].getThumbnailImage().width + "  " + _aggregations[i].getThumbnailImage().height );
      image( _aggregations[i].getThumbnailImage(), x, y, _agg_width, _agg_height );
      noFill();
      if ( i == _selected_aggregation )
      {
        stroke( AGGREGATION_VIEW_SELECTED_COLOR );
        strokeWeight( AGGREGATION_VIEW_SELECTED_OUTLINE_WEIGHT );
      }
      else
      {
        stroke( AGGREGATION_OUTLINE_COLOR );
        strokeWeight( AGGREGATION_VIEW_OUTLINE_WEIGHT );
      }
      rect( x, y, _agg_width, _agg_height );

      if ( i == _selected_aggregation )
      {
        textFont(_pixel_font_8); 
        fill( AGGREGATION_VIEW_SELECTED_COLOR );
      }
      else
      {
        textFont(_pixel_font_8); 
        fill( AGGREGATION_TEXT_COLOR );
      }
      textAlign( LEFT, BOTTOM );
      text( _aggregations[i].getID(), x+tx_offset, y+ty_offset+y_bottom_offset );
      textAlign( LEFT, TOP );
      text( _aggregations[i].getEmbryoID(), x+tx_offset, y+ty_offset + _agg_height + 3 );

      x += _agg_width + 2*_agg_border;
    }

    //
    // render the title
    //
    x = -30;
    y = 8;
    textFont( _verdana_font_16 );
    fill( TITLE_COLOR );
    textAlign( LEFT, BOTTOM );
    text( "Summaries", x+tx_offset, y+ty_offset+y_bottom_offset );
//    strokeWeight( 1 );
//    stroke(  KEY_COLOR );
//    //y -= 24;
//    x = 0;
//    line( x, y, textWidth("Computation View")-30, y );
//    line( x, y, x, _height );
  }

  void mouseReleasedSomewhere()
  {
    _over_x_scroll = false;
  }

  boolean mousePressedInView( int mx, int my )
  {
    if ( (my >= _scroll_pos[1]-_scroll_tick_width/2) && (my <= _scroll_pos[1]+_scroll_tick_width/2) )
    {
      if ( overXSlider( mx-_scroll_pos[0], my-_scroll_pos[1] ) )
      {
        _over_x_scroll = true;
        _over_x_scroll_pos = mx;
      }
    }
    else
    {
      int v = overAggregation(mx, my);
      if ( v != -1 )
      {
        _selected_aggregation = v;
        return true;
      }
    }

    return false;
  }

  void mouseDraggedInView( int mx, int my )
  {
    if ( _over_x_scroll )
    {
      int movement = mx-_over_x_scroll_pos;

      if ( abs(movement) > _scroll_tick_interval/2 )
      {
        if ( movement < 0 )
        {
          if ( (_start_render_index-1) >= 0 )
          {
            _start_render_index = _start_render_index-1;
            _over_x_scroll_pos = mx;
          }
        } 
        else
        {
          if (  (_start_render_index+1+_num_to_render) <= NUM_AGGREGATIONS )
          {
            _start_render_index = _start_render_index+1;
            _over_x_scroll_pos = mx;
          }
        }
      }
    }
  }

  boolean overXSlider( int mx, int my )
  {
    int min_pos = _scroll_pos[0] + _start_render_index*_scroll_tick_interval;
    int max_pos = _scroll_pos[0] + (_start_render_index+_num_to_render)*_scroll_tick_interval;
    if ( (mx >= min_pos) && (mx <= max_pos) ) return true;

    return false;
  }

  int overAggregation( int mx, int my )
  {
    int x = _agg_border;
    int y = _agg_view_border;
    for ( int i = _start_render_index; i < _start_render_index+_num_to_render; i++ )
    {
      if ( (mx >= x) && (mx <= x+_agg_width) && (my >= y) && (my <= y+_agg_height) ) return i;
      x += _agg_width + 2*_agg_border;
    }
    return -1;
  } 
}








