class ComputationGroupEmbryoView
{
  int _width, _height, _map_width, _map_height;
  int[] _map_pos, _text_offset;
  float[] _bbox_lower_left;
  float _coordinate_width, _coordinate_height;
  int _tick_length, _map_border;
  color[] _color_map;
  int[] _map_offset;

  Group _group;
  GroupElement _cell_of_interest;

  int _slider_width, _slider_height, _slider_border;
  int[] _slider_pos;
  float _slider_min, _slider_max, _slider_diff;
  String _slider_min_string, _slider_max_string;
  PImage _slider_colorbar_img;

  float _map_min_x, _map_max_x, _map_min_y, _map_max_y;

  int[] _x_scroll_pos, _y_scroll_pos;
  int _scroll_width, _scroll_offset, _x_scroll_length, _y_scroll_length;
  boolean _over_x_scroll, _over_y_scroll;
  int _over_x_scroll_pos, _over_y_scroll_pos;

  boolean _rolled_over_group_cell, _rolled_over_coi;
  int _group_cell_rolled_over;

  boolean _have_x_scroll, _have_y_scroll;

  int _plot_width, _plot_height;
  int[] _plot_pos, _plot_offset;
  int _plot_bar_width;
  int _map_plot_offset;
  float _plot_min_value, _plot_max_value;
  String _plot_min_string, _plot_max_string;
  boolean _pressed_near_plot;
  int[] _pressed_near_plot_start_pos, _pressed_near_plot_current_pos;

  int _pressed_cell_render_number;

  int _plot_num_to_render;
  int _plot_start_render_index;
  int _plot_scroll_tick_width, _plot_scroll_tick_interval, _plot_scroll_length;
  int[] _plot_scroll_pos;
  boolean _plot_over_x_scroll;
  int _plot_over_x_scroll_pos;

  ComputationGroupEmbryoView()
  {
    _bbox_lower_left = new float[2];
    _map_pos = new int[2];
    _map_offset = new int[2];

    _tick_length = EMBRYO_TICK_LENGTH;
    _map_border = EMBRYO_MAP_BORDER;

    _color_map = COLORMAP_2;

    _slider_pos = new int[2];
    _slider_width = COMP_GROUP_VIEW_SLIDER_WIDTH;
    _slider_border = COMP_GROUP_VIEW_SLIDER_BORDER;

    _rolled_over_group_cell = _rolled_over_coi = false;

    _map_min_x = _map_min_y = 0.0;
    _map_max_x = _map_max_y = 1.0;

    _x_scroll_pos = new int[2];
    _y_scroll_pos = new int[2];
    _scroll_width = SCROLL_WIDTH;
    _scroll_offset = SCROLL_OFFSET;

    _over_x_scroll = _over_y_scroll = false;
    _over_x_scroll_pos = _over_y_scroll_pos = 0;

    _have_x_scroll = _have_y_scroll = false;

    _plot_pos = new int[2];
    _plot_offset = new int[2];
    _plot_bar_width = COMP_GROUP_VIEW_PLOT_BAR_WIDTH;
    _map_plot_offset = COMP_GROUP_VIEW_PLOT_OFFSET;
    _plot_min_value = _plot_max_value = 0.0;

    _pressed_near_plot = false;
    _pressed_near_plot_start_pos = new int[2];
    _pressed_near_plot_current_pos = new int[2];

    _plot_num_to_render = _plot_start_render_index = 0;

    _plot_scroll_tick_width = SCROLL_WIDTH;
    _plot_scroll_tick_interval = _plot_scroll_length = 0;
    _plot_scroll_pos = new int[2];

    _plot_over_x_scroll = false;
    _plot_over_x_scroll_pos = 0;
  }

  int getPressedCellRenderNumber()
  {
    return _pressed_cell_render_number;
  }

  void setTextOffset( int tx, int ty )
  {
    _text_offset = new int[2];
    _text_offset[0] = tx;
    _text_offset[1] = ty;
  }

  void setDimensions( int w, int h )
  {
    _width = w;
    _height = h;
  }

  void initialize( Group group, GroupElement cell_of_interest )
  {
    _y_scroll_pos[0] = 0;

    _group = group;
    _cell_of_interest = cell_of_interest;

    // get the min and max coordinates of the group
    float[] min_pos, max_pos, pos;
    Embryo e;
    e = _embryos[embryoID2Index(_cell_of_interest.getEmbryoID())];
    pos = e.getCell(e.cellID2Index(_cell_of_interest.getCellID())).getPos();
    min_pos = new float[2];
    max_pos = new float[2];
    min_pos[0] = max_pos[0] = pos[0];
    min_pos[1] = max_pos[1] = pos[1];
    GroupElement ge;

    for ( int i = 0; i < _group.getNumElements(); i++ )
    {
      ge = _group.getElement(i);
      e = _embryos[embryoID2Index(ge.getEmbryoID())];
      pos = e.getCell(e.cellID2Index(ge.getCellID())).getPos();
      min_pos[0] = min( min_pos[0], pos[0] );
      min_pos[1] = min( min_pos[1], pos[1] );
      max_pos[0] = max( max_pos[0], pos[0] );
      max_pos[1] = max( max_pos[1], pos[1] );
    }

    // set the bounding box lower left corner coord
    _bbox_lower_left[0] = min_pos[0];
    _bbox_lower_left[1] = min_pos[1];

    // the width and height in coordinate space
    _coordinate_width = max_pos[0] - min_pos[0];
    _coordinate_height = max_pos[1] - min_pos[1];

    // dimensions in pixel space
    _map_width = min( round( _coordinate_width * (float)EMBRYO_MAP_WIDTH ), _width/2-_slider_width-_slider_border-_scroll_width-_scroll_offset-2*_map_border );
    _map_height = min( round( _coordinate_height * (float)EMBRYO_MAP_WIDTH ), _height-_scroll_width-_scroll_offset-3*_map_border/2 );

    // the size and position of the embryo map 
    _map_pos[0] = _y_scroll_pos[0] + _scroll_width + _scroll_offset + 2*_map_border;
    _map_pos[1] = _map_border;

    _map_offset[0] = (_width/2 - (_map_width+_slider_width+_slider_border+_scroll_width+_scroll_offset+2*_map_border))/2;
    _map_offset[1] = (_height - (_map_height+_scroll_width+_scroll_offset+3*_map_border/2))/2;

    //_map_offset[0] = _map_offset[1] = 0;

    _y_scroll_pos[1] = _map_pos[1];
    _x_scroll_pos[0] = _map_pos[0];
    _x_scroll_pos[1] = _map_pos[1]+_map_height+_scroll_offset+_map_border/2;
    _y_scroll_length = _map_height;
    _x_scroll_length = _map_width;

    // set the min and max of the window
    _map_min_x = _map_min_y = 0.0;
    _map_max_x = (float)_map_width / round(_coordinate_width * (float)EMBRYO_MAP_WIDTH);
    _map_max_y = (float)_map_height / round(_coordinate_height * (float)EMBRYO_MAP_WIDTH);

    _have_x_scroll = _have_y_scroll = false;
    if ( _map_max_x != 1.0 ) _have_x_scroll = true;
    if ( _map_max_y != 1.0 ) _have_y_scroll = true;

    // the color legend
    _slider_pos[0] = _map_pos[0]+_map_width+_slider_border;
    _slider_pos[1] = _map_pos[1];
    _slider_height = _map_height;

    // check if the group values are normalized
    if ( !group.valuesNormalized() ) group.normalizeValues();

    _slider_min = group.getMinValue();
    _slider_max = group.getMaxValue();
    _slider_diff = _slider_max - _slider_min;

    _slider_min_string = numToString( _slider_min );
    _slider_max_string = numToString( _slider_max );   

    buildColorBar();

    _plot_pos[0] = _width/2 + _map_plot_offset;
    _plot_pos[1] = _map_pos[1];
    _plot_width = min( (_width - _plot_pos[0]), group.getNumElements()*_plot_bar_width );
    _plot_height = _height-_scroll_width-_scroll_offset-3*_map_border/2;

    _plot_offset[0] = (_width/2-_map_plot_offset - _plot_width)/2;
    _plot_offset[1] = (_height-_plot_height-2*_map_border)/2;

    // get min and max and do floor and ceil to one tenth
    _plot_min_value = group.getMinValue();
    _plot_max_value = group.getMaxValue();

    _plot_min_value = floor(_plot_min_value*10.0)/10.0;
    _plot_max_value = ceil(_plot_max_value*10.0)/10.0;
    _plot_min_string = numToStringOneDecimal( _plot_min_value );
    _plot_max_string = numToStringOneDecimal( _plot_max_value );

    // compute the number of aggs that can be shown
    _plot_num_to_render = floor((float)_plot_width / (float)_plot_bar_width);
    _plot_start_render_index = min( _plot_start_render_index, (group.getNumElements()-_plot_num_to_render) );

    _plot_scroll_length = _plot_width;
    _plot_scroll_tick_interval = floor( (float)_plot_scroll_length / (float)group.getNumElements() );
    _plot_scroll_length = _plot_scroll_tick_interval*group.getNumElements();
    _plot_scroll_pos[0] = 0;
    _plot_scroll_pos[1] = _plot_height+_slider_border;
  }

  void buildColorBar()
  {
    _slider_colorbar_img = createImage( _slider_width, _slider_height, RGB );

    // set the colored pixels
    color c;
    for ( int i = 0; i < _slider_height; i++ )
    {
      c = getColor( _color_map, (float)i/(float)_slider_height );
      for ( int j = 0; j < _slider_width; j++ )
        _slider_colorbar_img.pixels[i*_slider_width+j] = c;
    }
  }


  int getViewWidth()
  {
    return _width;
  }

  int getViewHeight()
  {
    return _height;
  }

  void render( boolean name_rolled_over, int rolled_over_name )
  {
    if ( _group.getNumElements() == 0 ) return;

    int x, y, offset, tx_offset=0, ty_offset=0;

    noSmooth();

    pushMatrix();
    translate( _map_offset[0], _map_offset[1] );

    //
    // render the scroll bars
    //

    // y axis scroll bar
    if ( _have_y_scroll )
    {
      strokeWeight( SCROLL_LINE_WEIGHT );
      stroke( SCROLL_BACKGROUND_COLOR );    
      line( _y_scroll_pos[0], _y_scroll_pos[1], _y_scroll_pos[0]+_scroll_width, _y_scroll_pos[1] );
      line( _y_scroll_pos[0]+_scroll_width/2, _y_scroll_pos[1], _y_scroll_pos[0]+_scroll_width/2, _y_scroll_pos[1]+_y_scroll_length );
      line( _y_scroll_pos[0], _y_scroll_pos[1]+_y_scroll_length, _y_scroll_pos[0]+_scroll_width, _y_scroll_pos[1]+_y_scroll_length );

      strokeWeight( SCROLL_BAR_LINE_WEIGHT );
      if ( !_over_y_scroll ) stroke( SCROLL_COLOR );
      else stroke( SCROLL_SELECTED_COLOR );
      int max_pos = _y_scroll_pos[1]+(int)((1.0-_map_max_y)*(float)_y_scroll_length);
      int min_pos = _y_scroll_pos[1]+(int)((1.0-_map_min_y)*(float)_y_scroll_length);
      line( _y_scroll_pos[0]+_scroll_width/2, max_pos, _y_scroll_pos[0]+_scroll_width/2, min_pos );
    }

    // x axis scroll bar
    if ( _have_x_scroll )
    {
      strokeWeight( SCROLL_LINE_WEIGHT );
      stroke( SCROLL_BACKGROUND_COLOR ); 
      line( _x_scroll_pos[0], _x_scroll_pos[1], _x_scroll_pos[0], _x_scroll_pos[1]+_scroll_width );
      line( _x_scroll_pos[0], _x_scroll_pos[1]+_scroll_width/2, _x_scroll_pos[0]+_x_scroll_length, _x_scroll_pos[1]+_scroll_width/2 );
      line( _x_scroll_pos[0]+_x_scroll_length, _x_scroll_pos[1], _x_scroll_pos[0]+_x_scroll_length, _x_scroll_pos[1]+_scroll_width );

      strokeWeight( SCROLL_BAR_LINE_WEIGHT );
      if ( !_over_x_scroll ) stroke( SCROLL_COLOR );
      else stroke( SCROLL_SELECTED_COLOR );
      int min_pos = _x_scroll_pos[0]+(int)(_map_min_x*(float)_x_scroll_length);
      int max_pos = _x_scroll_pos[0]+(int)(_map_max_x*(float)_x_scroll_length);
      line( min_pos, _x_scroll_pos[1]+_scroll_width/2, max_pos, _x_scroll_pos[1]+_scroll_width/2 );
    }

    //
    // render the embryo map
    //
    if( _use_text_offset )
    {
      tx_offset = _text_offset[0] + _map_pos[0] + _map_offset[0];
      ty_offset = _text_offset[1] + _map_pos[1] + _map_offset[1];
    }
    int y_bottom_offset = 0;
    if ( !_use_text_offset )
    { 
      ty_offset = -PDF_OFFSET;
      y_bottom_offset = PDF_OFFSET-1;
    }

    pushMatrix();    
    translate( _map_pos[0], _map_pos[1] );

    fill( EMBRYO_MAP_BACKGROUND_COLOR );
    stroke( EMBRYO_MAP_OUTLINE_COLOR );
    strokeWeight( EMBRYO_MAP_OUTLINE_WEIGHT );
    rectMode( CORNER );
    rect( 0, 0, _map_width, _map_height );

    //
    // make the tick marks (assumes a domain of [0,1]).
    //  also assumes just 3 ticks per axis for now....
    //
    textFont(_pixel_font_8); 
    fill( EMBRYO_VIEW_TEXT_COLOR );
    stroke( EMBRYO_MAP_OUTLINE_COLOR );
    strokeWeight( EMBRYO_MAP_OUTLINE_WEIGHT );

    // the x-axis ticks
    textAlign( CENTER, TOP );
    offset = 1;
    x = 0;
    y = _map_height;
    line( x, y, x, y+_tick_length );
    String s = nf( _bbox_lower_left[0]+_map_min_x*_coordinate_width, 1, 2 );
    text( s, x+tx_offset, y+ty_offset+_tick_length+offset+2 );

    x += _map_width;
    line( x, y, x, y+_tick_length );
    s = nf( _bbox_lower_left[0]+_map_max_x*_coordinate_width, 1, 2 );
    text( s, x+tx_offset, y+ty_offset+_tick_length+offset+2 );

    // the y-axis ticks    
    textAlign( RIGHT, CENTER );
    x = 0;
    y = _map_height;
    line( x, y, x-_tick_length, y );
    s = nf( _bbox_lower_left[1]+_map_min_y*_coordinate_height, 1, 2 );
    text( s, x+tx_offset-_tick_length-offset-2, y+ty_offset );

    y -= _map_height;
    line( x, y, x-_tick_length, y );
    s = nf( _bbox_lower_left[1]+_map_max_y*_coordinate_height, 1, 2 );
    text( s, x+tx_offset-_tick_length-offset-2, y+ty_offset );

    //
    // render the cells
    //
    noFill();
    strokeWeight( EMBRYO_VIEW_CELL_STROKE_WEIGHT );
    float[] pos;
    float v;
    Embryo e;
    GroupElement ge;
    int cell_width;
    Cell c;

    // render the cell of interest if it is not rolled over    
    if ( !(name_rolled_over && (rolled_over_name == -1)) || !_rolled_over_coi)
    {
      e = _embryos[embryoID2Index(_cell_of_interest.getEmbryoID())];
      c = e.getCell(e.cellID2Index(_cell_of_interest.getCellID()));
      pos = c.getPos();

      if ( (pos[0] >= (_bbox_lower_left[0]+_map_min_x*_coordinate_width)) && (pos[0] <= (_bbox_lower_left[0]+_map_max_x*_coordinate_width)) && 
        (pos[1] >= (_bbox_lower_left[1]+_map_min_y*_coordinate_height)) && (pos[1] <= (_bbox_lower_left[1]+_map_max_y*_coordinate_height)) )
      {
        cell_width = EMBRYO_VIEW_CELL_WIDTH;
        fill( CURVEMAP_VIEW_SELECTED_COLOR );
        x = (int)((pos[0]-_bbox_lower_left[0]-_map_min_x*_coordinate_width)*(float)EMBRYO_MAP_WIDTH);
        y = _map_height - (int)((pos[1]-_bbox_lower_left[1]-_map_min_y*_coordinate_height)*(float)EMBRYO_MAP_WIDTH); // translate to bottom of map....

        if ( c.inCreatedGroup() ) 
        {
          stroke( EMBRYO_VIEW_IN_CREATED_GROUP_LINE_COLOR );
          rectMode( CENTER );
          rect( x, y, cell_width+2, cell_width+2 );
          rectMode( CORNER );
        }
        else 
        {
          stroke( CURVEMAP_VIEW_SELECTED_COLOR );
          ellipse( x, y, cell_width, cell_width );
        }
      }
    }

    // render the group
    int counter = 0;
    int cell_to_render_last=-1;
    for ( int i = 0; i < _group.getNumElements(); i++ )
    {
      ge = _group.getElement( i );
      e = _embryos[embryoID2Index(ge.getEmbryoID())];
      c = e.getCell(e.cellID2Index(ge.getCellID()));
      v = ge.getValue();
      e = _embryos[embryoID2Index(ge.getEmbryoID())];
      pos = c.getPos();

      if ( (pos[0] >= (_bbox_lower_left[0]+_map_min_x*_coordinate_width)) && (pos[0] <= (_bbox_lower_left[0]+_map_max_x*_coordinate_width)) && 
        (pos[1] >= (_bbox_lower_left[1]+_map_min_y*_coordinate_height)) && (pos[1] <= (_bbox_lower_left[1]+_map_max_y*_coordinate_height)) )
      {
        x = (int)((pos[0]-_bbox_lower_left[0]-_map_min_x*_coordinate_width)*(float)EMBRYO_MAP_WIDTH);
        y = _map_height - (int)((pos[1]-_bbox_lower_left[1]-_map_min_y*_coordinate_height)*(float)EMBRYO_MAP_WIDTH); // translate to bottom of map....

        cell_width = EMBRYO_VIEW_CELL_WIDTH;

        if ( ge.render() )
        {
          if ( (name_rolled_over && (counter == rolled_over_name)) || (_rolled_over_group_cell && (_group_cell_rolled_over == i)) )
          {
            cell_to_render_last = i;
          }
          else 
          {
            fill( getColor(_color_map, (1.0-ge.getNormalizedValue())) );
            if ( c.inCreatedGroup() ) 
            {
              stroke( EMBRYO_VIEW_IN_CREATED_GROUP_LINE_COLOR );
              rectMode( CENTER );
              rect( x, y, cell_width+2, cell_width+2 );
              rectMode( CORNER );
            }
            else
            {
              stroke( getColor(_color_map, (1.0-ge.getNormalizedValue())) );
              ellipse( x, y, cell_width, cell_width );
            }
          }
        }
        else
        {
          if ( _rolled_over_group_cell && (_group_cell_rolled_over == i) )
          {
            cell_to_render_last = i;
          }
          else
          {
            stroke( getColor(_color_map, (1.0-ge.getNormalizedValue())) );
            if ( c.inCreatedGroup() ) 
            {
              fill( EMBRYO_VIEW_IN_CREATED_GROUP_FILL_COLOR );
              rectMode( CENTER );
              rect( x, y, cell_width+2, cell_width+2 );
              rectMode( CORNER );
            }
            else
            {
              noFill();            
              ellipse( x, y, cell_width, cell_width );
            }
          }
        }
      }

      if ( ge.render() ) ++counter;
    }

    // render the rolled over cell
    if ( name_rolled_over )
    {
      cell_width = COMP_GROUP_VIEW_ROLLED_OVER_CELL_WIDTH;
      strokeWeight( EMBRYO_VIEW_ROLLED_OVER_CELL_WEIGHT );
      stroke( EMBRYO_VIEW_ROLLOVER_COLOR );

      if ( rolled_over_name == -1 )
      {
        fill( CURVEMAP_VIEW_SELECTED_COLOR );

        e = _embryos[embryoID2Index(_cell_of_interest.getEmbryoID())];
        pos = e.getCell(e.cellID2Index(_cell_of_interest.getCellID())).getPos();

        if ( (pos[0] >= (_bbox_lower_left[0]+_map_min_x*_coordinate_width)) && (pos[0] <= (_bbox_lower_left[0]+_map_max_x*_coordinate_width)) && 
          (pos[1] >= (_bbox_lower_left[1]+_map_min_y*_coordinate_height)) && (pos[1] <= (_bbox_lower_left[1]+_map_max_y*_coordinate_height)) )
        {
          x = (int)((pos[0]-_bbox_lower_left[0]-_map_min_x*_coordinate_width)*(float)EMBRYO_MAP_WIDTH);
          y = _map_height - (int)((pos[1]-_bbox_lower_left[1]-_map_min_y*_coordinate_height)*(float)EMBRYO_MAP_WIDTH); // translate to bottom of map....

          if ( !e.getCell(e.cellID2Index(_cell_of_interest.getCellID())).inCreatedGroup() )  
            ellipse( x, y, cell_width, cell_width );
          else
          {
            rectMode( CENTER );
            rect( x, y, cell_width+2, cell_width+2 );
            rectMode( CORNER );
          }
        }
      }
      else
      {     
        if ( cell_to_render_last != -1 )
        {  
          ge = _group.getElement( cell_to_render_last );
          v = ge.getValue();
          e = _embryos[embryoID2Index(ge.getEmbryoID())];
          pos = e.getCell(e.cellID2Index(ge.getCellID())).getPos();

          if ( (pos[0] >= (_bbox_lower_left[0]+_map_min_x*_coordinate_width)) && (pos[0] <= (_bbox_lower_left[0]+_map_max_x*_coordinate_width)) && 
            (pos[1] >= (_bbox_lower_left[1]+_map_min_y*_coordinate_height)) && (pos[1] <= (_bbox_lower_left[1]+_map_max_y*_coordinate_height)) )
          {
            x = (int)((pos[0]-_bbox_lower_left[0]-_map_min_x*_coordinate_width)*(float)EMBRYO_MAP_WIDTH);
            y = _map_height - (int)((pos[1]-_bbox_lower_left[1]-_map_min_y*_coordinate_height)*(float)EMBRYO_MAP_WIDTH); // translate to bottom of map....

            fill( getColor(_color_map, (1.0-ge.getNormalizedValue())) );

            if ( !e.getCell(e.cellID2Index(ge.getCellID())).inCreatedGroup() )
              ellipse( x, y, cell_width, cell_width );
            else
            {
              rectMode( CENTER );
              rect( x, y, cell_width+2, cell_width+2 );
              rectMode( CORNER );
            }
          }
        } 
      }     
    }
    else if ( _rolled_over_group_cell )
    {
      if ( cell_to_render_last != -1 )
      {
        cell_width = EMBRYO_VIEW_ROLLED_OVER_CELL_WIDTH;
        strokeWeight( EMBRYO_VIEW_ROLLED_OVER_CELL_WEIGHT );
        stroke( EMBRYO_VIEW_ROLLOVER_COLOR );

        ge = _group.getElement( cell_to_render_last );
        v = ge.getValue();
        e = _embryos[embryoID2Index(ge.getEmbryoID())];
        pos = e.getCell(e.cellID2Index(ge.getCellID())).getPos();

        if ( (pos[0] >= (_bbox_lower_left[0]+_map_min_x*_coordinate_width)) && (pos[0] <= (_bbox_lower_left[0]+_map_max_x*_coordinate_width)) && 
          (pos[1] >= (_bbox_lower_left[1]+_map_min_y*_coordinate_height)) && (pos[1] <= (_bbox_lower_left[1]+_map_max_y*_coordinate_height)) )
        {
          x = (int)((pos[0]-_bbox_lower_left[0]-_map_min_x*_coordinate_width)*(float)EMBRYO_MAP_WIDTH);
          y = _map_height - (int)((pos[1]-_bbox_lower_left[1]-_map_min_y*_coordinate_height)*(float)EMBRYO_MAP_WIDTH); // translate to bottom of map....

          if ( ge.render() )
            fill( getColor(_color_map, (1.0-ge.getNormalizedValue())) );
          else
            noFill();

          if ( !e.getCell(e.cellID2Index(ge.getCellID())).inCreatedGroup() )
            ellipse( x, y, cell_width, cell_width );
          else
          {
            rectMode( CENTER );
            rect( x, y, cell_width+2, cell_width+2 );
            rectMode( CORNER );
          }
        }
      }
    }
    else if ( _rolled_over_coi )
    {
      cell_width = EMBRYO_VIEW_ROLLED_OVER_CELL_WIDTH;
      strokeWeight( EMBRYO_VIEW_ROLLED_OVER_CELL_WEIGHT );
      stroke( EMBRYO_VIEW_ROLLOVER_COLOR );

      fill( CURVEMAP_VIEW_SELECTED_COLOR );

      e = _embryos[embryoID2Index(_cell_of_interest.getEmbryoID())];
      pos = e.getCell(e.cellID2Index(_cell_of_interest.getCellID())).getPos();

      if ( (pos[0] >= (_bbox_lower_left[0]+_map_min_x*_coordinate_width)) && (pos[0] <= (_bbox_lower_left[0]+_map_max_x*_coordinate_width)) && 
        (pos[1] >= (_bbox_lower_left[1]+_map_min_y*_coordinate_height)) && (pos[1] <= (_bbox_lower_left[1]+_map_max_y*_coordinate_height)) )
      {
        x = (int)((pos[0]-_bbox_lower_left[0]-_map_min_x*_coordinate_width)*(float)EMBRYO_MAP_WIDTH);
        y = _map_height - (int)((pos[1]-_bbox_lower_left[1]-_map_min_y*_coordinate_height)*(float)EMBRYO_MAP_WIDTH); // translate to bottom of map....

        if ( !e.getCell(e.cellID2Index(_cell_of_interest.getCellID())).inCreatedGroup() )
          ellipse( x, y, cell_width, cell_width );
        else
        {
          rectMode( CENTER );
          rect( x, y, cell_width+2, cell_width+2 );
          rectMode( CORNER );
        }
      }
    }

    // render the name of the embryo map
    /*
    if ( _group.getNumElements() != 0 )
    {
      textFont(_pixel_font_8); 
      x = 0;
      y = -4;
      fill( EMBRYO_VIEW_TEXT_COLOR );
      textAlign( LEFT, BOTTOM );
      text( _group.getElement(0).getEmbryoID(), x+tx_offset, y+ty_offset );
    }*/

    popMatrix();

    //
    // render the slider
    //
    if( _use_text_offset )
    {
      tx_offset = _text_offset[0] + _slider_pos[0] + _map_offset[0];
      ty_offset = _text_offset[1] + _slider_pos[1] + _map_offset[1];
    }

    pushMatrix();    
    translate( _slider_pos[0], _slider_pos[1] );

    textFont(_pixel_font_8); 

    // render the colormap
    image( _slider_colorbar_img, 0, 0, _slider_width, _slider_height );

    // outline the slider bar
    stroke( EMBRYO_MAP_OUTLINE_COLOR );
    strokeWeight( EMBRYO_MAP_OUTLINE_WEIGHT );
    noFill();
    rect( 0, 0, _slider_width, _slider_height );

    // write the min and max values
    fill( EMBRYO_VIEW_TEXT_COLOR );
    stroke( EMBRYO_MAP_OUTLINE_COLOR );
    strokeWeight( EMBRYO_MAP_OUTLINE_WEIGHT );

    x = _slider_width;
    y = 0;
    line( x, y, x+_tick_length, y );
    textAlign( LEFT, BOTTOM );
    text( _slider_max_string, x+tx_offset+_tick_length, y+ty_offset+y_bottom_offset );

    y += _slider_height;
    line( x, y, x+_tick_length, y );
    textAlign( LEFT, TOP );
    text( _slider_min_string, x+tx_offset+_tick_length, y+ty_offset+3 );

    //
    // finished with the slider
    //
    popMatrix();

    // done with the map
    popMatrix();

    //
    // render the plot
    //
    if( _use_text_offset )
    {
      tx_offset = _text_offset[0] + _plot_pos[0]+_plot_offset[0];
      ty_offset = _text_offset[1] + _plot_pos[1]+_plot_offset[1];
    }

    pushMatrix();
    translate( _plot_pos[0]+_plot_offset[0], _plot_pos[1]+_plot_offset[1] );
    
    // the title
    x = _plot_width+2;
    y = 0;
    textFont(_pixel_font_8);
    textAlign( RIGHT, BOTTOM );
    fill( EMBRYO_VIEW_TEXT_COLOR );
    text( "metric value", x+tx_offset-_tick_length, y+ty_offset+y_bottom_offset-2 );

    noStroke();
    fill(  COMP_GROUP_VIEW_PLOT_BACKGROUND_COLOR );
    rect( 0, 0, _plot_width, _plot_height );
    
    counter = 0;
    for( int i = 0; i < _plot_start_render_index; i++ )
    {
      ge = _group.getElement( i );
      if ( ge.render() ) ++counter;
    }

    x = 0;
    y = _plot_height;
    for ( int i = _plot_start_render_index; i < _plot_start_render_index+_plot_num_to_render; i++ )
    {
      ge = _group.getElement( i );
      v = ge.getValue();

      if ( ge.render() ) 
      {
        if ( (name_rolled_over && (counter == rolled_over_name)) || (_rolled_over_group_cell && (_group_cell_rolled_over == i)) )
          fill( CURVEMAP_VIEW_OVERLAY_PLOT_ROLLED_OVERLINE_COLOR );
        else
          fill( COMP_GROUP_VIEW_PLOT_SELECTED_COLOR );
        ++counter;
      }
      else fill( COMP_GROUP_VIEW_PLOT_FILL_COLOR );

      rect( x, y, _plot_bar_width, -(v-_plot_min_value)/(_plot_max_value-_plot_min_value)*_plot_height );
      x += _plot_bar_width;
    }

    noFill();
    stroke( COMP_GROUP_VIEW_PLOT_OUTLINE_COLOR );
    rect( -1, -1, _plot_width+1, _plot_height+1 );

    fill( EMBRYO_VIEW_TEXT_COLOR );
    stroke( COMP_GROUP_VIEW_PLOT_OUTLINE_COLOR );
    strokeWeight( EMBRYO_MAP_OUTLINE_WEIGHT );

    x = 0;
    y = -1;
    line( x, y, x-_tick_length, y );
    textAlign( RIGHT, CENTER );
    text( _plot_max_string, x+tx_offset-_tick_length, y+ty_offset+1 );

    y += _plot_height+1;
    line( x, y, x-_tick_length, y );
    textAlign( RIGHT, CENTER );
    text( _plot_min_string, x+tx_offset-_tick_length, y+ty_offset );

    //
    // render the scroll
    //
    if ( _plot_num_to_render != _group.getNumElements() )
    {
      noFill();
      stroke( SCROLL_BACKGROUND_COLOR );
      strokeWeight(  SCROLL_LINE_WEIGHT );
      line( _plot_scroll_pos[0], _plot_scroll_pos[1], _plot_scroll_pos[0]+_plot_scroll_length, _plot_scroll_pos[1] );

      x = _plot_scroll_pos[0];
      y = _plot_scroll_pos[1]-_plot_scroll_tick_width/2;
      line( x, y, x, y+_plot_scroll_tick_width );
      x += _plot_scroll_length;
      line( x, y, x, y+_plot_scroll_tick_width );

      // and the scroll bar
      if ( !_plot_over_x_scroll ) stroke( SCROLL_COLOR );
      else stroke( SCROLL_SELECTED_COLOR );
      strokeWeight(  SCROLL_BAR_LINE_WEIGHT );
      int min_pos = _plot_scroll_pos[0] + _plot_start_render_index*_plot_scroll_tick_interval;
      int max_pos = _plot_scroll_pos[0] + (_plot_start_render_index+_plot_num_to_render)*_plot_scroll_tick_interval;
      line( min_pos, _plot_scroll_pos[1], max_pos, _plot_scroll_pos[1] );
    }

    //
    // done with the plot
    //
    popMatrix();

    //
    // render the selection bounding box
    //
    if ( _pressed_near_plot )
    {
      noFill();
      stroke( COMP_GROUP_VIEW_PLOT_BBOX_COLOR );
      strokeWeight( EMBRYO_MAP_OUTLINE_WEIGHT );
      rectMode( CORNERS );
      rect( _pressed_near_plot_start_pos[0], _pressed_near_plot_start_pos[1], _pressed_near_plot_current_pos[0], _pressed_near_plot_current_pos[1] );
      rectMode( CORNER );
    }
  }

  void mouseMovedInView( int mx, int my )
  {
    reinitFlags();
    if ( (mx >= _map_pos[0]+_map_offset[0]) && (mx <= (_map_pos[0]+_map_width+_map_offset[0])) && (my >= _map_pos[1]+_map_offset[1]) && (my <= (_map_pos[1]+_map_height+_map_offset[1])) )
    {
      rolledOverCell( mx, my );
      _comp_group_map_hover = true;
    }
  }

  void reinitFlags()
  {
    _rolled_over_group_cell = _rolled_over_coi = false;
  }

  boolean mousePressedInView( int mx, int my, boolean right_pressed )
  {
    _pressed_near_plot = false;

    if ( (mx >= _plot_pos[0]-_map_plot_offset) )
    {
      if ( (my >= (_plot_pos[1]+_plot_offset[1]+_plot_scroll_pos[1]-_plot_scroll_tick_width/2)) && (my <= (_plot_pos[1]+_plot_offset[1]+_plot_scroll_pos[1]+_plot_scroll_tick_width/2)) )
      {
        if ( overPlotXSlider( mx-_plot_pos[0]-_plot_offset[0], my-_plot_pos[1]-_plot_offset[1] ) )
        {
          _plot_over_x_scroll = true;
          _plot_over_x_scroll_pos = mx;
        }
      }
      else
      {
        _pressed_near_plot = true;
        _pressed_near_plot_start_pos[0] = _pressed_near_plot_current_pos[0] = mx;
        _pressed_near_plot_start_pos[1] = _pressed_near_plot_current_pos[1] = my;
      }
    }
    else if ( (mx >= _y_scroll_pos[0]+_map_offset[0]) && (mx <= (_y_scroll_pos[0]+_scroll_width+_map_offset[0])) )
    {
      if ( overYSlider( mx-_y_scroll_pos[0]-_map_offset[0], my-_y_scroll_pos[1]-_map_offset[1] ) )
      {
        _over_y_scroll = true;
        _over_y_scroll_pos = my;
      }
    }
    else if ( (my >= _x_scroll_pos[1]+_map_offset[1]) && (my <= (_x_scroll_pos[1]+_slider_width+_map_offset[1])) )
    {
      if ( overXSlider( mx-_x_scroll_pos[0]-_map_offset[0], my-_x_scroll_pos[1]-_map_offset[1] ) )
      {
        _over_x_scroll = true;
        _over_x_scroll_pos = mx;
      }
    }
    else
      return pressedCell( mx, my, right_pressed );

    return false;
  }

  boolean overPlotXSlider( int mx, int my )
  {
    int min_pos = _plot_scroll_pos[0] + _plot_start_render_index*_plot_scroll_tick_interval;
    int max_pos = _plot_scroll_pos[0] + (_plot_start_render_index+_plot_num_to_render)*_plot_scroll_tick_interval;
    if ( (mx >= min_pos) && (mx <= max_pos) ) return true;

    return false;
  }

  void mouseDraggedInView( int mx, int my, boolean right_pressed )
  {
    if ( _over_y_scroll )
    {
      float movement = (float)(my - _over_y_scroll_pos)/(float)_y_scroll_length;
      _map_max_y -= movement;
      _map_min_y -= movement;

      if ( _map_max_y > 1.0 )
      {
        movement = _map_max_y - 1.0;
        _map_max_y = 1.0;
        _map_min_y -= movement;
      }
      if ( _map_min_y < 0.0 )
      {
        movement = 0.0-_map_min_y;
        _map_min_y = 0.0;
        _map_max_y += movement;
      }

      _over_y_scroll_pos = my;
    }
    else if ( _over_x_scroll )
    {
      float movement = (float)(mx-_over_x_scroll_pos)/(float)_x_scroll_length;
      _map_max_x += movement;
      _map_min_x += movement;

      if ( _map_max_x > 1.0 )
      {
        movement = _map_max_x - 1.0;
        _map_max_x = 1.0;
        _map_min_x -= movement;
      }
      if ( _map_min_x < 0.0 )
      {
        movement = 0.0-_map_min_x;
        _map_min_x = 0.0;
        _map_max_x += movement;
      }

      _over_x_scroll_pos = mx;
    }
    else if ( right_pressed )
    {
      pressedCell( mx, my, right_pressed );
    }
    else if ( _pressed_near_plot )
    {
      _pressed_near_plot_current_pos[0] = mx;
      _pressed_near_plot_current_pos[1] = my;
    }
    else if ( _plot_over_x_scroll )
    {
      int movement = mx-_plot_over_x_scroll_pos;

      if ( abs(movement) > _plot_scroll_tick_interval/2 )
      {
        if ( movement < 0 )
        {
          if ( (_plot_start_render_index-1) >= 0 )
          {
            _plot_start_render_index = _plot_start_render_index-1;
            _plot_over_x_scroll_pos = mx;
          }
        } 
        else
        {
          if (  (_plot_start_render_index+1+_plot_num_to_render) <= _group.getNumElements() )
          {
            _plot_start_render_index = _plot_start_render_index+1;
            _plot_over_x_scroll_pos = mx;
          }
        }
      }
    }
  }

  boolean mouseReleasedSomewhere()
  {
    _over_y_scroll = _over_x_scroll = false;
    _plot_over_x_scroll = false;
    if ( _pressed_near_plot )
    {
      int x_start = min( max(_pressed_near_plot_start_pos[0] - _plot_pos[0] - _plot_offset[0],0), _plot_width )/_plot_bar_width;
      int x_stop = min( max(_pressed_near_plot_current_pos[0] - _plot_pos[0] - _plot_offset[0],0), _plot_width )/_plot_bar_width;
      for ( int i = min(x_start,x_stop)+_plot_start_render_index; i < max(x_start,x_stop)+_plot_start_render_index; i++ )
        _group.getElement(i).render(true);
      _pressed_near_plot = false;
      return true;  
    }
    return false;
  }

  boolean overYSlider( int mx, int my )
  {
    int min_pos = (int)((1.0-_map_min_y)*(float)_y_scroll_length);
    int max_pos = (int)((1.0-_map_max_y)*(float)_y_scroll_length);
    if ( (my >= max_pos) && (my <= min_pos) ) return true;

    return false;
  }

  boolean overXSlider( int mx, int my )
  {
    int min_pos = (int)(_map_min_x*(float)_x_scroll_length);
    int max_pos = (int)(_map_max_x*(float)_x_scroll_length);
    if ( (mx >= min_pos) && (mx <= max_pos) ) return true;

    return false;
  }

  void rolledOverCell( int mx, int my )
  {
    mx -= (_map_pos[0]+_map_offset[0]);
    my -= (_map_pos[1]+_map_offset[1]);

    _rolled_over_group_cell = _rolled_over_coi = false;

    // first check out the group
    GroupElement ge;
    Embryo e;
    float[] pos;
    int x, y;
    for ( int i = 0; i < _group.getNumElements(); i++ )
    {
      ge = _group.getElement( i );
      e = _embryos[embryoID2Index(ge.getEmbryoID())];
      pos = e.getCell(e.cellID2Index(ge.getCellID())).getPos();

      x = (int)((pos[0]-_bbox_lower_left[0]-_map_min_x*_coordinate_width)*(float)EMBRYO_MAP_WIDTH);
      y = _map_height - (int)((pos[1]-_bbox_lower_left[1]-_map_min_y*_coordinate_height)*(float)EMBRYO_MAP_WIDTH); // translate to bottom of map....

      if ( (abs(x-mx) <= (EMBRYO_VIEW_CELL_HALF_WIDTH+1)) && (abs(y-my) <= (EMBRYO_VIEW_CELL_HALF_WIDTH+1)) )
      {
        _rolled_over_group_cell = true;
        _group_cell_rolled_over = i;
        return;
      }
    }

    // check if the cell of interest is over
    e = _embryos[embryoID2Index(_cell_of_interest.getEmbryoID())];
    pos = e.getCell(e.cellID2Index(_cell_of_interest.getCellID())).getPos();
    x = (int)((pos[0]-_bbox_lower_left[0])*(float)EMBRYO_MAP_WIDTH);
    y = _map_height-(int)((pos[1]-_bbox_lower_left[1])*(float)EMBRYO_MAP_WIDTH); 

    if ( (abs(x-mx) <= (EMBRYO_VIEW_CELL_HALF_WIDTH+1)) && (abs(y-my) <= (EMBRYO_VIEW_CELL_HALF_WIDTH+1)) )
      _rolled_over_coi = true;
  }

  boolean rolledOverGroupCell()
  { 
    return _rolled_over_group_cell;
    //return false;
  }

  boolean rolledOverCOI()
  {
    return _rolled_over_coi;
  }

  int groupCellRolledOver()
  {
    return _group_cell_rolled_over;
  }

  boolean pressedCell( int mx, int my, boolean right_pressed )
  {
    mx -= (_map_pos[0]+_map_offset[0]);
    my -= (_map_pos[1]+_map_offset[1]);

    // first check out the group
    GroupElement ge;
    Embryo e;
    float[] pos;
    int x, y;
    int counter = 0;
    for ( int i = 0; i < _group.getNumElements(); i++ )
    {
      ge = _group.getElement( i );
      e = _embryos[embryoID2Index(ge.getEmbryoID())];
      pos = e.getCell(e.cellID2Index(ge.getCellID())).getPos();

      x = (int)((pos[0]-_bbox_lower_left[0]-_map_min_x*_coordinate_width)*(float)EMBRYO_MAP_WIDTH);
      y = _map_height - (int)((pos[1]-_bbox_lower_left[1]-_map_min_y*_coordinate_height)*(float)EMBRYO_MAP_WIDTH); // translate to bottom of map....

      if ( (abs(x-mx) <= (EMBRYO_VIEW_CELL_HALF_WIDTH+1)) && (abs(y-my) <= (EMBRYO_VIEW_CELL_HALF_WIDTH+1)) )
      {
        if ( !right_pressed )
        {
          ge.render( true );
          _pressed_cell_render_number = counter;

          return true;
        }
        else
        {
          Cell c = e.getCell(e.cellID2Index(ge.getCellID()));
          if ( !c.inCreatedGroup() )
          {
            c.inCreatedGroup(true);
            GroupElement ge2 = new GroupElement( e.getID(), c.getID() );
            ge2.render( true );
            _created_group.addElement( ge2 );
          }
        }
      }

      if ( ge.render() ) ++counter;
    }
    return false;
  }
}




































