
class Cell 
{
  int _id;  // assumes numeric id for now!
  float[] _pos;
  float[][] _profile;
  String _embryo_id;
  float[][] _profile_min_max;
  boolean _in_created_group;
  int _num_in_created_group;

  Cell( int id, float[] pos, float[][] profile, String e_id )
  {
    _pos = new float[2];
    _profile = new float[NUM_GENES][NUM_TIMESTEPS];
    _profile_min_max = new float[NUM_GENES][2];
    setValues( id, pos, profile, e_id );
    _in_created_group = false;
    _num_in_created_group = 0;
  }
  
  Cell( Cell other )
  {
    
    _pos = new float[2];
    _profile = new float[NUM_GENES][NUM_TIMESTEPS];
    _profile_min_max = new float[NUM_GENES][2];
    setValues( other._id, other._pos, other._profile, other._embryo_id );
    _in_created_group = other._in_created_group;
    _num_in_created_group = other._num_in_created_group;
  }
  
  void setValues( int id, float[] pos, float[][] profile, String e_id )
  {
    _id = id;

    _pos[0] = pos[0];
    _pos[1] = pos[1];
    
    _embryo_id = e_id;

    for( int i = 0; i < NUM_GENES; i++ )
      for( int j = 0; j < NUM_TIMESTEPS; j++ )
        _profile[i][j] = profile[i][j];
    setMinMax();
  }
  
  void setMinMax()
  {
    for ( int i = 0; i < NUM_GENES; i++ )
    {
      _profile_min_max[i][0] = Float.MAX_VALUE;
      _profile_min_max[i][1] = Float.MIN_VALUE;
      for( int j = 0; j < NUM_TIMESTEPS; j++ )
      {
        _profile_min_max[i][0] = min( _profile_min_max[i][0], _profile[i][j] );
        _profile_min_max[i][1] = max( _profile_min_max[i][1], _profile[i][j] );
      }
    }
 
  }
  
  int getID()
  {
    return _id;
  }
  
  float[] getPos()
  {
    return _pos;
  }
  
  String getEmbryoID()
  {
    return _embryo_id;
  }
  
  float getProfileValue( int g, int t )
  {
    return _profile[g][t];
  }
  
  float[] getTimeSieresMinMax( int g )
  {
    return _profile_min_max[g];
  }
  
  void inCreatedGroup( boolean i )
  { 
    if ( i ) ++_num_in_created_group;
    else _num_in_created_group = max( _num_in_created_group-1, 0 );
    
    if ( i ) _in_created_group = i;
    else if ( _num_in_created_group == 0 ) _in_created_group = i;
    else _in_created_group = true;
  }
  
  boolean inCreatedGroup()
  {
    return _in_created_group;
  }
  
  void printProfile()
  {
    println( _embryo_id + "  " + _id );
    for ( int i = 0; i < NUM_GENES; i++ )
    {
      print( _gene_names[i] + ":  " );
      for ( int j = 0; j < NUM_TIMESTEPS; j++ )
        print( _profile[i][j] + "  ");
      println();
    }
    println();
  }
}

/************************************************************************/

class Embryo
{
  String _id;
  int _num_cells, _add_cell_at;
  Cell[] _cells;
  
  Embryo( String id, int num_cells )
  {
    _id = id;
    _num_cells = num_cells;
    _cells = new Cell[_num_cells];
    
    _add_cell_at = 0;
  }
  
  Embryo( Embryo other )
  {
    _id = other._id;
    _num_cells = other._num_cells;
    _cells = new Cell[_num_cells];
    for ( int i = 0; i < _num_cells; i++ )
      _cells[i] = new Cell( other._cells[i] );
      
    _add_cell_at = other._add_cell_at;
  }
  
  void addCell( Cell c )
  {
    if ( _add_cell_at < _num_cells ) 
    {
      _cells[_add_cell_at] = new Cell(c);
      ++_add_cell_at;
    }
    else println( "Trying to add a cell past array size in embryo " + _id );
  }
  
  int getNumCells()
  {
    return _num_cells;
  }
  
  String getID()
  {
    return _id;
  }
  
  Cell getCell( int index )
  {
    return _cells[index];
  }
  
  // find the array index for a specific cell id; this will
  //  need to be made more generic in the future; currently 
  //  assumes that the cells have a numeric id, in the range
  //  [1,num_cells], and ordered
  int cellID2Index( int id )
  {
    return id-1;
  }
    
}

int embryoID2Index( String id )
{
  for ( int i = 0; i < NUM_EMBRYOS; i++ )
    if ( id.equals(_embryos[i]._id) ) return i;
    
  println( "Embryo " + id + " not found." );
  return -1;
}
