//
// CONFIG FILE
//
void readConfigFile( String file )
{
  String[] rows = loadStrings( file );

  int counter = 0;
  String[] cols;
  ArrayList embryo_files = new ArrayList();
  ArrayList aggregation_files = new ArrayList();
  
  boolean colormap_set = false; 
  
  while ( counter < rows.length )
  {
    cols = splitTokens( rows[counter] );  

    if ( cols.length == 0 )
    {
    }
    else if ( cols[0].equals("GENE_NAMES:") )
    {
      NUM_GENES =  cols.length-1;
      _gene_names = new String[NUM_GENES];
      for ( int i = 0; i < NUM_GENES; i++ )
        _gene_names[i] = cols[i+1];
    } 

    else if ( cols[0].equals("TIMESTEP_NAMES:") )
    {
      NUM_TIMESTEPS = cols.length-1;
      _timestep_names = new String[NUM_TIMESTEPS];
      for ( int i = 0; i < NUM_TIMESTEPS; i++ )
        _timestep_names[i] = cols[i+1];
    } 

    else if ( cols[0].equals("EMBRYO:") )
      embryo_files.add( cols[1] );

    else if( cols[0].equals("COMPUTATION:") )
      aggregation_files.add( cols[1] );
      
    else if( cols[0].equals("GROUP:") )
      _initial_group_files.add( cols[1] );
      
    else if( cols[0].equals("COLORMAP:") && (cols.length >= 2) )
    {
      if ( cols[1].equals("GnBu") )
        _selected_colormap = COLORMAP_1;
      else if ( cols[1].equals("YlGnBu") )
        _selected_colormap = COLORMAP_2;
      else if ( cols[1].equals("YlOrRd") )
        _selected_colormap = COLORMAP_3;
      else if ( cols[1].equals("YlGn") )
        _selected_colormap = COLORMAP_4;
      else if ( cols[1].equals("YlOrBr") )
        _selected_colormap = COLORMAP_5;
      else if ( cols[1].equals("BrBG") )
        _selected_colormap = COLORMAP_6;
      else
        _selected_colormap = COLORMAP_2;
        
      colormap_set = true;
        
      if ( (cols.length == 3) && cols[2].equalsIgnoreCase("invert") )
        _invert_colormap = true;
    } 

    ++counter;
  }
  
   if (!colormap_set) _selected_colormap = COLORMAP_2;


  // create the embryos
  NUM_EMBRYOS = embryo_files.size();
  _embryos = new Embryo[NUM_EMBRYOS];
  for ( int i = 0; i < NUM_EMBRYOS; i++ )
    _embryos[i] = new Embryo( readEmbryoFile( (String)embryo_files.get(i) ) );

  // create the aggregations
  NUM_AGGREGATIONS = aggregation_files.size();
  _aggregations = new Aggregation[NUM_AGGREGATIONS];
  for ( int i = 0; i < NUM_AGGREGATIONS; i++ )
    _aggregations[i] = new Aggregation( readAggregationFile( (String)aggregation_files.get(i) ) );
}


//
// EMBRYO FILE
//
Embryo readEmbryoFile( String file )
{
  String[] rows = loadStrings( file );
  String[] cols;

  // discard the comments
  int counter = 0;
  while ( splitTokens(rows[counter],",")[0].substring(0,2).equals( "##" ) )
    ++counter;

  // get the embryo ID
  String id = splitTokens(rows[counter],",:")[1].trim();

  // get the number of cells in the embryo
  ++counter;
  int num_cells = parseInt(splitTokens(rows[counter],",:")[1].trim());

  // create the embryo
  println( "Creating the " + id + " embryo with " + num_cells + " cells." );
  Embryo e = new Embryo( id, num_cells );

  // get the indices for the expression profiles
  ++counter;
  int[][] indices = new int[NUM_GENES][NUM_TIMESTEPS];
  String name;
  for ( int i = 0; i < NUM_GENES; i++ )
    for ( int j = 0; j < NUM_TIMESTEPS; j++ )
    {
      name = _gene_names[i] + "__" + _timestep_names[j];

      indices[i][j] = -1;      
      cols = splitTokens( rows[counter], "," );
      for ( int k = 0; k < cols.length; k++ )
        if ( cols[k].equals(name) ) 
        {
          indices[i][j] = k;
          break;
        }
    }

  // read the profiles
  Cell c;
  int cell_id;
  float[] pos = new float[2];
  float[][] profile = new float[NUM_GENES][NUM_TIMESTEPS];
  for ( int k = 0; k < num_cells; k++ )
  {
    ++counter;

    // make sure we still have rows to read!
    //
    // NOTE: we need to do something about the profiles here....
    if ( counter == rows.length ) 
    {
      println( "Not enough cell rows in " + file );
      break;
    } 

    cols = splitTokens( rows[counter], "," );
    cell_id = parseInt( cols[0] );
    pos[0] = parseFloat( cols[4] );
    pos[1] = parseFloat( cols[5] );

    // now fill the cell's profile in the appropriate order
    for ( int i = 0; i < NUM_GENES; i++ )
      for ( int j = 0; j < NUM_TIMESTEPS; j++ )
      {
        if ( indices[i][j] != -1 ) 
        {
          if ( !cols[indices[i][j]].equalsIgnoreCase("nan") )
            profile[i][j] = parseFloat( cols[indices[i][j]] ); 
          else 
            profile[i][j] = Float.NaN;
        }
        else
          profile[i][j] = Float.NaN;
      }

    c = new Cell( cell_id, pos, profile, id );
    e.addCell( c );
  }

  return e;
}


//
// AGGREGATION FILE
//
Aggregation readAggregationFile( String file )
{
  String[] rows = loadStrings( file );
  String[] cols;

  // discard the comments
  int counter = 0;
  while ( splitTokens(rows[counter],",")[0].substring(0,2).equals( "##" ) )
    ++counter;

  // get the associated species
  String e_id = splitTokens(rows[counter], ",:")[1].trim();
  
  // get the metric name
  ++counter;
  String id = splitTokens(rows[counter], ",:")[1].trim();

  // get the aggregation ID
  ++counter;
  String a_id = splitTokens(rows[counter], ",:")[1].trim();
  
  // get the number of cells in the embryo
  int num_cells = _embryos[embryoID2Index(e_id)].getNumCells();

  // create the embryo
  println( "Creating the " + id + " aggregation of the " + e_id + " embryo with " + num_cells + " cells." );
  Aggregation a = new Aggregation( id, e_id, a_id, num_cells );

  // read in the scores and computation groups for each cell
  counter += 2;
  GroupElement ge;
  int c_id, size_cg;
  float v;
  for ( int i = 0; i < num_cells; i++ )
  {
    // make sure we still have rows to read!
    //
    // NOTE: we need to do something about the groups here....
    if ( counter == rows.length ) 
    {
      println( "Not enough cell rows in " + file );
      println( rows.length );
      break;
    } 

    cols = splitTokens( rows[counter], "," );

    // get the cell info
    c_id = parseInt( cols[0] );
    v = parseFloat( cols[1] );
    ge = new GroupElement( e_id, c_id, v );

    // get the computational group info
    size_cg = parseInt( cols[2] );
    int g_counter = 3;
    String other_e_id;
    GroupElement other_ge;
    Group cg = new Group( "computation group for cell " + c_id + " in embryo " + e_id );
    for ( int j = 0; j < size_cg; j++ )
    {
      // make sure we still have cells to read!
      //
      // NOTE: we need to do something about the group here....
      if ( g_counter == cols.length )
      {
        println( "Not enough cells in " + file + " at row " + counter);
        break;
      } 
      
      other_e_id = cols[g_counter];
      c_id = parseInt( cols[g_counter+1] );
      v = parseFloat( cols[g_counter+2] );
      other_ge = new GroupElement( other_e_id, c_id, v );
      
      cg.addElement( other_ge );
      
      g_counter += 3;
    }
    
    a.addCell( ge, cg );

    ++counter;
  }

  return a;
}



























